(* parser.sml *)
(* driver for CSX parser - derived from the pascal parser example *)

structure Parser =
struct

structure CSXLrVals = CSXLrValsFun(structure Token = LrParser.Token)

structure CSXLex = CSXLexFun(structure Tokens = CSXLrVals.Tokens)

structure CSXParser = Join(structure Lex= CSXLex
		              structure LrParser = LrParser
		              structure ParserData = CSXLrVals.ParserData
                      structure AST = DataTypes)

open DataTypes;


val lexing_error = ref false 

exception SyntaxError;

fun parse s =
    let val dev = TextIO.openIn s
	val stream = CSXParser.makeLexer(fn i => TextIO.inputN(dev,i))
	fun error (e,i:int,_) = (
            lexing_error := true;
	    TextIO.output(TextIO.stdOut,
                          s ^ "," ^ " line " ^ (Int.toString i) ^ ", Error: " ^ e ^ "\n")
        )
        val ast = CSXParser.parse(0,stream,error,()) 
     in
        if !lexing_error then
                raise SyntaxError
        else 
                ast
    end
    handle LrParser.ParseError => raise SyntaxError
    handle IO.Io{cause, function, name} => ( raise cause)
    handle e => (print("Unknown lexer/parser error \n"); raise e)

fun lex filename =
    let val file = TextIO.openIn filename
        val lexer = CSXLex.makeLexer(fn i => TextIO.inputN(file, i))
        fun do_it() =
                let val t = lexer()
                 in t
        end in 
         do_it()
    end 

end (* structure Parser *)
